*
*		PROGRAM MULTIDYNSIMPIELDV
*	
*		3/28/17
*		Andrew Q. Philips
*		Texas A&M University
* -------------------------------------------------------------------------
* -------------------------------------------------------------------------
* NOTES: --interaction is only suitable for dichotomous variables.
*		 --interaction variable is interacted with ALL variables (including
*		   dummy variables)
*		 --if dummyset not given, dummy will be set to 0
*		 --this is the lagged dependent variable version of program
*		PANELDYNSIMPIEINTER
*		 --added "restrictldv" option, which when specified means that the
*		 --LDV is not interacted w/ the interaction var. 
* -------------------------------------------------------------------------


capture program drop multidynsimpieldv
capture program define multidynsimpieldv , rclass
syntax [varlist] [if] [in], [ dvs(varlist) shockvar(varname) 	  ///
Time(numlist integer > 1) SHock(numlist) shockvar2(varname) shock2(numlist) ///
dummy(varlist) dummyset(numlist) sig(numlist integer < 100)]		  ///
[saving(string)] [INTERaction(varname)]	  ///
[INTYpe(string)] [id(varname)] [restrictldv shockvar3(varname) shock3(numlist) shockvar4(varname) shock4(numlist)] ///
 [shockvar5(varname) shock5(numlist) shockvar6(varname) shock6(numlist)]		///
 [shockvar7(varname) shock7(numlist) shockvar8(varname) shock8(numlist) 		///
 shockvar9(varname) shock9(numlist) shockvar10(varname) shock10(numlist)		///
 shockvar11(varname) shock11(numlist) shockvar12(varname) shock12(numlist)	  ///
 shockvar13(varname) shock13(numlist)]
 								 

set seed 234509
version 12
marksample touse
preserve


 if "`sig'" != ""	{						// getting the CI's signif
 	local signif `sig'
 }
 else	{
 	local signif 95
 }
 local sigl = (100-`signif')/2
 local sigu = 100-((100-`signif')/2)

 *if "`range'" != "" {						// How far to simulate?
 *	local range `range'
 *}
 *else {
 	local range 20
 *}
 if `time' >= `range' {
 	di in r _n "The range of simulation must be longer than the shock time"
 	exit 198
 }
 if "`dvs'" == ""	{
 	di in r _n "Must specify dependent compositional variables in dvs( )"
 	exit 198
 }
 if "`shockvar'" == ""	{
 	di in r _n "A shockvar, not included in [varlist], must be specified"
 	exit 198
 }
 if "`intype'" == "" & "`interaction'" != "" {
 	di in r _n "Intype must either be (on) or (off) in intype"
 	exit 198
 } 


 * ------------------------Generating Interaction Variables  ---------------------
 loc interIVs
 loc Ishockvarable
 loc intershockvariables
 loc dumIs
 qui if "`interaction'" != ""	{
	 
	 loc constituentI `interaction'						// constituent term for I-var
	 
	 foreach var of varlist `varlist'	{				// interaction * IVs
		tempvar I`var'
		bysort `id' : gen `I`var'' = `var'*`interaction' 
	 	local interIVs `"`interIVs' `I`var''"'
	 }
	 
	 tempvar Ishockvar									// interaction * shockvar
	 bysort `id' : gen `Ishockvar' = `shockvar'*`interaction'
	 loc Ishockvarable `"`Ishockvar'"'
	   
	 forv i = 2/13	{								// interaction * other shockvars
		if "`shockvar`i''" != ""	{
	 		tempvar Ishockvar`i'
			bysort `id': gen `Ishockvar`i'' = `shockvar`i''*`interaction'
			loc intershockvariables `"`intershockvariables' `Ishockvar`i''"'
	 	}
	 }
	 										
	 if "`dummy'" != ""	{							// interaction * dummies
	 	foreach var in `dummy'	{
			tempvar dumI`var'
			bysort `id' : gen `dumI`var'' = `var'*`interaction'
			loc dumIs `"`dumIs' `dumI`var''"'
		}
	 }
	 else	{
	 	loc dumIs ""
	 } 
	 
}
 
* ------------------------Generate LDV & run model --------------------------------
loc model
local i 1
qui foreach var of varlist `dvs'	{
	tempvar ldepvar`i'
	bysort `id': gen `ldepvar`i'' = l.`var'
	if "`interaction'" != ""	{
		if "`restrictldv'" != ""	{			// if we don't want an I*LDV
		}
		else	{
			tempvar LIdepvar`i'
			bysort `id' `timeset': gen `LIdepvar`i'' = `ldepvar`i''*`interaction'
		}
	}
	* get the model:
	local model `"`model' (`var' `ldepvar`i'' `LIdepvar`i'' `varlist' `interIVs' `shockvar' `Ishockvarable' `shockvar2' `shockvar3' `shockvar4' `shockvar5' `shockvar6' `shockvar7' `shockvar8' `shockvar9' `shockvar10' `shockvar11' `shockvar12' `shockvar13' `intershockvariables' `dummy' `dumIs' `constituentI')"'
	local i = `i' + 1
}
local maxdv = `i' - 1					// need # of compositions
qui sureg `model'       
qui keep if e(sample)
noi estsimp sureg `model'
      
* ------------------------ Scalars and Setx ---------------------
qui setx mean						// set everything to means to start       

qui su `shockvar', meanonly     	// scalars for shock
local sv = r(mean)
local vs = `sv' + `shock'   
qui setx `shockvar' mean

qui forv i = 2/13	{				// and the additional shocks
	if "`shockvar`i''" != ""	{
		su `shockvar`i'', meanonly
		local sv`i' = r(mean)
		local vs`i' = `sv`i'' + `shock`i''
		setx `shockvar`i'' mean
	}
}         
          
qui if "`interaction'" != ""	{
	if "`intype'" == "on" {			// set interacted shock var & others
		setx `constituentI' 1		// constituent term is "on"
		setx `Ishockvar' `sv'
		forv i = 2/13	{
			if "`shockvar`i''" != ""	{
				setx `Ishockvar`i'' `sv`i''
			}
		}
		foreach var of varlist `varlist'	{
			su `var', meanonly
			local `var'_m = r(mean)
			setx `I`var'' (``var'_m')
		}
	}
	else	{						// if intype is off
		setx `constituentI' 0		// constituent term is "off"
		setx `Ishockvar' 0
		forv i = 2/13	{
			if "`shockvar`i''" != ""	{
				setx `Ishockvar`i'' 0
			}
		}
		foreach var of varlist `varlist'	{
			setx `I`var'' 0
		}
	}
}

local i 1
qui foreach var of varlist `dvs'	{	// set lagged DVs to their means
	su `var', meanonly
	scalar m`i' = r(mean)
	setx `ldepvar`i'' m`i'
	if "`interaction'" != ""	{
		if "`restrictldv'" != ""	{
		}
		else	{
			if "`intype'" == "on"	{
				setx `LIdepvar`i'' m`i'
			}
			else	{
				setx `LIdepvar`i'' 0
			}
		}
	}
	local i = `i' + 1
}

qui setx (`varlist') mean				// double check that IVs are at means

local i 1								// set the dummies
qui if "`dummy'" != ""	{
	if "`dummyset'" != ""	{
		foreach var in `dummy'	{
			local m 1
			foreach k of numlist `dummyset'	{
				if `m' == `i'	{
					setx `var' `k'
				}
				local m = `m' + 1
			}
			local i = `i' + 1
		}
	}
	else	{
		setx(`dummy') 0
	}
}

local i 1								// and interactive dummies
qui if "`dummy'" != ""	{
	if "`interaction'" != ""	{
		foreach var in `dummy'	{
			if "`interaction'" != ""	{
				if "`intype'" == "on"	{
					local m 1
					foreach k of numlist `dummyset'	{
						if `m' == `i' & "`k'" == "1"	{
							setx(`dumI`var'') 1	// dummy interaction ON
						}
						local m = `m' + 1
					}
				}
				else	{
					setx(`dumI`var'') 0			// dummy interaction OFF
				}
			}
			local i = `i' + 1
		}
	}
}

* ------------------------ Predict Values, t = 1 ----------------------------------
local preddv
loc i 1
qui foreach var of varlist `dvs'	{
	tempvar t`i'log1
	loc preddv `"`preddv' `t`i'log1' "'
	loc i = `i' + 1	
}
qui simqi, ev genev(`preddv')

loc denominator1
loc i 1
qui foreach var in `dvs'	{
	su `t`i'log1', meanonly
	scalar m`i' = r(mean)			// these scalars become the new LDV
	loc denominator1 `"`denominator1' + (exp(`t`i'log1'))"'
	loc i = `i' + 1
}

 * ------------------------Loop Through Time-------------------------------------
di ""
nois _dots 0, title(Simulation in Progress...please wait) reps(`range')
qui forv i = 2/`range'	{
	noi _dots `i' 0
	
	forv x = 1/`maxdv'	{				// set new value of LDV and lagged inter DV
		setx `ldepvar`x'' (m`x')
		if "`interaction'" != ""	{
			if "`restrictldv'" != ""	{
			}
			else	{
				if "`intype'" == "on"	{
					setx `LIdepvar`x'' m`x'
				}
				else	{
					setx `LIdepvar`x'' 0
				}
			}
		}
		
	}
	
	* set dummy variables:
	local op 1
	qui if "`dummy'" != ""	{
		if "`dummyset'" != ""	{
			foreach var in `dummy'	{
				local m 1 
				foreach k of numlist `dummyset'	{
					if `m' == `op'	{
						setx `var' `k'
					}
					local m = `m' + 1
				}
				local op = `op' + 1
			}
		}
		else	{
			setx(`dummy') 0
		}
	}
	local op 1
	qui if "`dummy'" != ""	{	// set interactive dummies to mean of shock, if they exist & if dummy = 1
		if "`interaction'" != ""	{
			foreach var in `dummy'	{
				if "`interaction'" != ""	{
					if "`intype'" == "on"	{
						local m 1
						foreach k of numlist `dummyset'	{
							if `m' == `op' & "`k'" == "1"	{
								setx(`dumI`var'') 1		// dummy interaction ON
							}
							local m = `m' + 1
						}
					}
					else	{
						setx(`dumI`var'') 0				// Dummy interaction OFF
					}
				}
				local op = `op' + 1
			}
		}
	}
	 
	qui setx `shockvar' mean 
	qui forv z = 2/13	{				// and the additional shocks
		if "`shockvar`z''" != ""	{
			setx `shockvar`z'' mean
		}
	} 
	qui if "`interaction'" != ""	{
		if "`intype'" == "on"	{
			setx `constituentI' 1		// constituent term is "on"
			setx `Ishockvar' `sv'	// set interacted shock var
			forv z = 2/13	{
				if "`shockvar`z''" != ""	{
					setx `Ishockvar`z'' `sv`z''
				}
			}
		}
		else	{
			setx `constituentI' 0		// constituent term is "off"
			setx `Ishockvar' 0
			forv z = 2/13	{
				if "`shockvar`z''" != ""	{
					setx `Ishockvar`z'' 0
				}
			}
		}
	}
	
	qui if `i' >= `time'	{	// we experience shock at t, which stays in system
		setx `shockvar'	(`vs')		// vs = shock + mean of shockvar
		forv l = 2/13	{
			if "`shockvar`l''" != ""	{
				setx `shockvar`l'' (`vs`l'')
			}
		}
		if "`interaction'" != ""	{
			if "`intype'" == "on"	{
				setx `Ishockvar' `vs'
				forv l = 2/13	{
					if "`shockvar`l''" != ""	{
						setx `Ishockvar`l'' (`vs`l'')
					}
				}
			}
			else	{
				setx `Ishockvar' 0
				forv l = 2/13	{
					if "`shockvar`l''" != ""	{
						setx `Ishockvar`l'' 0
					}
				}
			}
		}
	} 		// close `i' >= time
	
	qui setx (`varlist') mean
	if "`interaction'" != ""	{
		foreach var of varlist `varlist'	{
			if "`intype'" == "on"	{
				setx `I`var'' (``var'_m')
			}
			else	{
				setx `I`var'' 0
			}
		}
	}

	local preddv
	qui forv x = 1/`maxdv'	{
		tempvar t`x'log`i'
		local preddv `"`preddv' `t`x'log`i'' "'
	}
	simqi, ev genev(`preddv')				// get new predictions
	local denominator`i'
	qui forv x = 1/`maxdv'	{
		su `t`x'log`i'', meanonly
		scalar m`x' = r(mean)
		local denominator`i' `"`denominator`i'' + (exp(`t`x'log`i''))"' // need this for below
	}	
}	// close time loop

* ------------------------Un-Transform------------------------------------------
local keepthese
local z = `maxdv' + 1
qui forv i = 1/`range'	{
	qui forv m = 1/`maxdv'	{
		tempvar var`m'_pie`i'
		gen `var`m'_pie`i'' = (exp(`t`m'log`i''))/(1 `denominator`i'')
		_pctile `var`m'_pie`i'', p(`sigl',`sigu')			// grab CIs for each DV
		gen var`m'_pie_ll_`i' = r(r1)
		gen var`m'_pie_ul_`i' = r(r2)
		loc keepthese `"`keepthese' var`m'_pie_ll_`i' var`m'_pie_ul_`i' "'
	}
	tempvar var`z'_pie`i'				// for the untransformation baseline
	gen `var`z'_pie`i'' = 1/(1 `denominator`i'')
	_pctile `var`z'_pie`i'', p(`sigl',`sigu')
	gen var`z'_pie_ll_`i' = r(r1)
	gen var`z'_pie_ul_`i' = r(r2)
	loc keepthese `"`keepthese' var`z'_pie_ll_`i' var`z'_pie_ul_`i' "'
}

keep `keepthese'
qui keep in 1
tempvar count
qui gen `count' = _n

local reshapevar
local maxvars = `maxdv' + 1				// get max # of DVs
qui forv i = 1/`maxvars'	{
	local reshapevar `"`reshapevar' var`i'_pie_ll_ var`i'_pie_ul_ "'
}

qui reshape long `reshapevar', j(time) i(`count')
qui drop `count'

qui forv i = 1/`maxvars'	{
	gen mid`i' = (var`i'_pie_ll_ + var`i'_pie_ul_)/2
}

di ""
if "`saving'" != ""	{
	noi save `saving'.dta, replace
}
else	{
	noi save dynsimpie_results.dta, replace
}

end 										// end
* ----------------------------------------------------------------------------